# encoding: utf-8
'''
Created on 13 juin 2018

@author: denis
'''
import paho.mqtt.client as mqtt
import json

"""
Wildcard Subscriptions

Wild cards can be used in destination names when subscribing as a consumer.
This allows you to subscribe to multiple destinations or hierarchy of destinations.
    / is used to separate names in a path
    + is used to match any name in a path
    # is used to recursively match path names

        For example using the above, these subscriptions are possible
        PRICE/# : Any price for any product on any exchange
        PRICE/STOCK/# : Any price for a stock on any exchange
        PRICE/STOCK/NASDAQ/+ : Any stock price on NASDAQ
        PRICE/STOCK/+/IBM : Any IBM stock price on any exchange
"""


class ClientMQTTBase(mqtt.Client):
    def __init__(self, topics, host='localhost', port=1883, keepalive=60, username=None, password=None):
        super(ClientMQTTBase, self).__init__()
        
        self.host = 'loraserver.tetraneutral.net'
        #self.host = host
        self.port = 1883
        self.keepalive = keepalive
        if username:
            self.username_pw_set(username, password)
        self.topics = topics

    def sub_topics(self):
        return [(topic, qos) for topic, qos in self.topics]

    def unsub_topics(self):
        return [topic for topic, _ in self.topics]

    def startMQTT(self):
        self.connect(self.host, self.port, self.keepalive)
        self.subscribe(self.sub_topics())

    def stopMQTT(self):
        self.unsubscribe(self.unsub_topics())
        self.disconnect()

    def publish_message(self, topic, payload=None, qos=0, retain=False):
        try:
            message = json.dumps(payload)

            print("ClientMQTTBase::publish_message: %s %s", topic, message)
            self.publish(topic, message, qos, retain)

        except Exception as e:
            print("ClientMQTTBase::publish_message error: %s", e)

    def on_connect(self, mqttc, obj, flags, rc):
        print("on_connect:", rc)

    def on_message(self, mqttc, obj, msg):
        print("on_message:", msg.topic, msg.qos, msg.payload)

    def on_publish(self, mqttc, obj, mid):
        print("on_publish:", mid, obj)

    def on_subscribe(self, mqttc, obj, mid, granted_qos):
        print("on_subscribe:", mid, granted_qos)

    def on_unsubscribe(self, client, userdata, mid):
        print("on_unsubscribe:", mid, userdata)

    def on_disconnect(self, client, userdata, mid):
        print("on_disconnect:", mid, userdata)

    def on_log(self, mqttc, obj, level, string):
        print("onlog:", level, string)

#
